@@ -15,6 +15,8 @@ from grid2op.Space import GridObjects
 # TODO see if it can be done in c++ easily
 class ValueStore:
     """
+    INTERNAL USE ONLY
+
     .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
 
     """
@@ -191,6 +193,7 @@ class _BackendAction(GridObjects):
         self.prod_v = ValueStore(self.n_gen, dtype=dt_float)
         self.load_p = ValueStore(self.n_load, dtype=dt_float)
         self.load_q = ValueStore(self.n_load, dtype=dt_float)
+        self.storage_power = ValueStore(self.n_storage, dtype=dt_float)
 
         self.activated_bus = np.full((self.n_sub, 2), dtype=dt_bool, fill_value=False)
         self.big_topo_to_subid = np.repeat(list(range(self.n_sub)), repeats=self.sub_info)
@@ -210,8 +213,9 @@ class _BackendAction(GridObjects):
         self._gens_bus = None
         self._lines_or_bus = None
         self._lines_ex_bus = None
+        self._storage_bus = None
 
-    def reorder(self, no_load, no_gen, no_topo, no_shunt):
+    def reorder(self, no_load, no_gen, no_topo, no_storage, no_shunt):
         """
         .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
 
@@ -226,6 +230,9 @@ class _BackendAction(GridObjects):
         self.prod_v.reorder(no_gen)
         self.load_p.reorder(no_load)
         self.load_q.reorder(no_load)
+
+        self.storage_power.reorder(no_storage)
+
         if self.shunts_data_available:
             self.shunt_p.reorder(no_shunt)
             self.shunt_q.reorder(no_shunt)
@@ -243,6 +250,10 @@ class _BackendAction(GridObjects):
         self.prod_v.reset()
         self.load_p.reset()
         self.load_q.reset()
+        self.storage_power.reset()
+        # storage unit have their power reset to 0. each step
+        self.storage_power.changed[:] = True
+        self.storage_power.values[:] = 0.
 
         # shunts
         if self.shunts_data_available:
@@ -262,6 +273,7 @@ class _BackendAction(GridObjects):
         self.prod_v.all_changed()
         self.load_p.all_changed()
         self.load_q.all_changed()
+        self.storage_power.all_changed()
 
         # TODO handle shunts
         # shunts
@@ -281,7 +293,7 @@ class _BackendAction(GridObjects):
 
         Parameters
         ----------
-        other: grid2op.Action.BaseAction.BaseAction
+        other: :class:`grid2op.Action.BaseAction.BaseAction`
 
         Returns
         -------
@@ -294,6 +306,7 @@ class _BackendAction(GridObjects):
         set_topo_vect = other._set_topo_vect
         switcth_topo_vect = other._change_bus_vect
         redispatching = other._redispatch
+        storage_power = other._storage_power
 
         # I deal with injections
         # Ia set the injection
@@ -310,10 +323,15 @@ class _BackendAction(GridObjects):
             if "prod_v" in dict_injection:
                 tmp = dict_injection["prod_v"]
                 self.prod_v.set_val(tmp)
+
         # Ib change the injection aka redispatching
         if other._modif_redispatch:
             self.prod_p.change_val(redispatching)
 
+        # Ic storage unit
+        if other._modif_storage:
+            self.storage_power.set_val(storage_power)
+
         # II shunts
         if self.shunts_data_available:
             shunts = {}
@@ -381,7 +399,7 @@ class _BackendAction(GridObjects):
         return self
 
     def __call__(self):
-        injections = self.prod_p, self.prod_v, self.load_p, self.load_q
+        injections = self.prod_p, self.prod_v, self.load_p, self.load_q, self.storage_power
         topo = self.current_topo
         shunts = None
         if self.shunts_data_available:
@@ -413,6 +431,12 @@ class _BackendAction(GridObjects):
         self._lines_ex_bus.copy_from_index(self.current_topo, self.line_ex_pos_topo_vect)
         return self._lines_ex_bus
 
+    def get_storages_bus(self):
+        if self._storage_bus is None:
+            self._storage_bus = ValueStore(self.n_storage, dtype=dt_int)
+        self._storage_bus.copy_from_index(self.current_topo, self.storage_pos_topo_vect)
+        return self._storage_bus
+
     def _get_active_bus(self):
         self.activated_bus[:] = False
         tmp = self.current_topo.values-1
