@@ -33,10 +33,12 @@ except (ImportError, ModuleNotFoundError):
 
 class PandaPowerBackend(Backend):
     """
+    INTERNAL
+
     .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
 
         If you want to code a backend to use grid2op with another powerflow, you can get inspired
-        from this class. Note However that implies knowning the behaviour
+        from this class. Note However that implies knowing the behaviour
         of PandaPower.
 
     This module presents an example of an implementation of a `grid2op.Backend` when using the powerflow
@@ -108,6 +110,7 @@ class PandaPowerBackend(Backend):
         self.load_pu_to_kv = None
         self.lines_or_pu_to_kv = None
         self.lines_ex_pu_to_kv = None
+        self.storage_pu_to_kv = None
 
         self.p_or = None
         self.q_or = None
@@ -122,6 +125,10 @@ class PandaPowerBackend(Backend):
         self.load_q = None
         self.load_v = None
 
+        self.storage_p = None
+        self.storage_q = None
+        self.storage_v = None
+
         self.prod_p = None
         self.prod_q = None
         self.prod_v = None
@@ -146,7 +153,6 @@ class PandaPowerBackend(Backend):
         self.cst_1 = dt_float(1.0)
         self._topo_vect = None
         self.slack_id = None
-        self.comp_time = 0.
 
         # function to rstore some information
         self.__nb_bus_before = None  # number of substation in the powergrid
@@ -170,8 +176,41 @@ class PandaPowerBackend(Backend):
             self._apply_trafo_lv
         ]
 
+        self.tol = None  # this is NOT the pandapower tolerance !!!! this is used to check if a storage unit
+        # produce / absorbs anything
+
+        # TODO storage doc (in grid2op rst) of the backend
+        self.can_output_theta = True  # I support the voltage angle
+        self.theta_or = None
+        self.theta_ex = None
+        self.load_theta = None
+        self.gen_theta = None
+        self.storage_theta = None
+
+    def get_theta(self):
+        """
+        TODO doc
+
+        Returns
+        -------
+        theta_or: ``numpy.ndarray``
+            For each orgin side of powerline, gives the voltage angle (in degree)
+        theta_ex: ``numpy.ndarray``
+            For each extremity side of powerline, gives the voltage angle (in degree)
+        load_theta: ``numpy.ndarray``
+            Gives the voltage angle (in degree) to the bus at which each load is connected
+        gen_theta: ``numpy.ndarray``
+            Gives the voltage angle (in degree) to the bus at which each generator is connected
+        storage_theta: ``numpy.ndarray``
+            Gives the voltage angle (in degree) to the bus at which each storage unit is connected
+        """
+        return self.cst_1 * self.theta_or,  self.cst_1 * self.theta_ex, self.cst_1 * self.load_theta, \
+               self.cst_1 * self.gen_theta, self.cst_1 * self.storage_theta
+
     def get_nb_active_bus(self):
         """
+        INTERNAL
+
         .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
 
         Compute the amount of buses "in service" eg with at least a powerline connected to it.
@@ -201,6 +240,8 @@ class PandaPowerBackend(Backend):
 
     def reset(self, path=None, grid_filename=None):
         """
+        INTERNAL
+
         .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
 
         Reload the grid.
@@ -217,6 +258,8 @@ class PandaPowerBackend(Backend):
 
     def load_grid(self, path=None, filename=None):
         """
+        INTERNAL
+
         .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
 
         Load the _grid, and initialize all the member of the class. Note that in order to perform topological
@@ -246,7 +289,9 @@ class PandaPowerBackend(Backend):
         i_ref = None
         self._iref_slack = None
         self._id_bus_added = None
-        pp.runpp(self._grid, numba=numba_)
+        with warnings.catch_warnings():
+            warnings.filterwarnings("ignore")
+            pp.runpp(self._grid, numba=numba_)
         if np.all(~self._grid.gen["slack"]):
             # there are not defined slack bus on the data, i need to hack it up a little bit
             pd2ppc = self._grid._pd2ppc_lookups["bus"]  # pd2ppc[pd_id] = ppc_id
@@ -273,7 +318,9 @@ class PandaPowerBackend(Backend):
         else:
             self.slack_id = np.where(self._grid.gen["slack"])[0]
 
-        pp.runpp(self._grid, numba=numba_)
+        with warnings.catch_warnings():
+            warnings.filterwarnings("ignore")
+            pp.runpp(self._grid, numba=numba_)
         self.__nb_bus_before = self._grid.bus.shape[0]
         self.__nb_powerline = self._grid.line.shape[0]
         self._init_bus_load = self.cst_1 * self._grid.load["bus"].values
@@ -283,8 +330,8 @@ class PandaPowerBackend(Backend):
 
         t_for = self.cst_1 * self._grid.trafo["hv_bus"].values
         t_fex = self.cst_1 * self._grid.trafo["lv_bus"].values
-        self._init_bus_lor = np.concatenate((self._init_bus_lor, t_for)).astype(np.int)
-        self._init_bus_lex = np.concatenate((self._init_bus_lex, t_fex)).astype(np.int)
+        self._init_bus_lor = np.concatenate((self._init_bus_lor, t_for)).astype(dt_int)
+        self._init_bus_lex = np.concatenate((self._init_bus_lex, t_fex)).astype(dt_int)
 
         self._grid["ext_grid"]["va_degree"] = 0.0
 
@@ -322,12 +369,34 @@ class PandaPowerBackend(Backend):
         if "name" in self._grid.load.columns and not self._grid.load["name"].isnull().values.any():
             self.name_load = [nl for nl in self._grid.load["name"]]
         else:
-            self.name_load = ["load_{bus}_{index_gen}".format(**row, index_gen=i)
+            self.name_load = ["load_{bus}_{index_load}".format(**row, index_load=i)
                               for i, (_, row) in enumerate(self._grid.load.iterrows())]
         self.name_load = np.array(self.name_load)
+
+        self.n_storage = copy.deepcopy(self._grid.storage.shape[0])
+        if self.n_storage == 0:
+            self.set_no_storage()
+        else:
+            if "name" in self._grid.storage.columns and not self._grid.storage["name"].isnull().values.any():
+                self.name_storage = [nl for nl in self._grid.storage["name"]]
+            else:
+                self.name_storage = ["storage_{bus}_{index_sto}".format(**row, index_sto=i)
+                                     for i, (_, row) in enumerate(self._grid.storage.iterrows())]
+            self.name_storage = np.array(self.name_storage)
+
         self.n_sub = copy.deepcopy(self._grid.bus.shape[0])
         self.name_sub = ["sub_{}".format(i) for i, row in self._grid.bus.iterrows()]
         self.name_sub = np.array(self.name_sub)
+
+        # "hack" to handle topological changes, for now only 2 buses per substation
+        add_topo = copy.deepcopy(self._grid.bus)
+        add_topo.index += add_topo.shape[0]
+        add_topo["in_service"] = False
+        self._grid.bus = pd.concat((self._grid.bus, add_topo))
+
+        self._init_private_attrs()
+
+    def _init_private_attrs(self):
         #  number of elements per substation
         self.sub_info = np.zeros(self.n_sub, dtype=dt_int)
 
@@ -341,14 +410,13 @@ class PandaPowerBackend(Backend):
         self.line_or_to_sub_pos = np.zeros(self.n_line, dtype=dt_int)
         self.line_ex_to_sub_pos = np.zeros(self.n_line, dtype=dt_int)
 
+        if self.n_storage > 0:
+            self.storage_to_subid = np.zeros(self.n_storage, dtype=dt_int)
+            self.storage_to_sub_pos = np.zeros(self.n_storage, dtype=dt_int)
+
         pos_already_used = np.zeros(self.n_sub, dtype=dt_int)
         self._what_object_where = [[] for _ in range(self.n_sub)]
 
-        # self._grid.line.sort_index(inplace=True)
-        # self._grid.trafo.sort_index(inplace=True)
-        # self._grid.gen.sort_index(inplace=True)
-        # self._grid.load.sort_index(inplace=True)
-
         for i, (_, row) in enumerate(self._grid.line.iterrows()):
             sub_or_id = int(row["from_bus"])
             sub_ex_id = int(row["to_bus"])
@@ -401,6 +469,16 @@ class PandaPowerBackend(Backend):
 
             self._what_object_where[sub_id].append(("load", "bus", i))
 
+        if self.n_storage > 0:
+            for i, (_, row) in enumerate(self._grid.storage.iterrows()):
+                sub_id = int(row["bus"])
+                self.sub_info[sub_id] += 1
+                self.storage_to_subid[i] = sub_id
+                self.storage_to_sub_pos[i] = pos_already_used[sub_id]
+                pos_already_used[sub_id] += 1
+
+                self._what_object_where[sub_id].append(("storage", "bus", i))
+
         self.dim_topo = np.sum(self.sub_info)
         self._compute_pos_big_topo()
 
@@ -408,21 +486,16 @@ class PandaPowerBackend(Backend):
         self._corresp_name_fun = {}
 
         self._get_vector_inj = {}
-        self._get_vector_inj["load_p"] = self._load_grid_load_p_mw #lambda grid: grid.load["p_mw"]
-        self._get_vector_inj["load_q"] = self._load_grid_load_q_mvar #lambda grid: grid.load["q_mvar"]
-        self._get_vector_inj["prod_p"] = self._load_grid_gen_p_mw #lambda grid: grid.gen["p_mw"]
-        self._get_vector_inj["prod_v"] = self._load_grid_gen_vm_pu #lambda grid: grid.gen["vm_pu"]
-
-        # "hack" to handle topological changes, for now only 2 buses per substation
-        add_topo = copy.deepcopy(self._grid.bus)
-        add_topo.index += add_topo.shape[0]
-        add_topo["in_service"] = False
-        self._grid.bus = pd.concat((self._grid.bus, add_topo))
+        self._get_vector_inj["load_p"] = self._load_grid_load_p_mw  # lambda grid: grid.load["p_mw"]
+        self._get_vector_inj["load_q"] = self._load_grid_load_q_mvar  # lambda grid: grid.load["q_mvar"]
+        self._get_vector_inj["prod_p"] = self._load_grid_gen_p_mw  # lambda grid: grid.gen["p_mw"]
+        self._get_vector_inj["prod_v"] = self._load_grid_gen_vm_pu  # lambda grid: grid.gen["vm_pu"]
 
         self.load_pu_to_kv = self._grid.bus["vn_kv"][self.load_to_subid].values.astype(dt_float)
         self.prod_pu_to_kv = self._grid.bus["vn_kv"][self.gen_to_subid].values.astype(dt_float)
         self.lines_or_pu_to_kv = self._grid.bus["vn_kv"][self.line_or_to_subid].values.astype(dt_float)
         self.lines_ex_pu_to_kv = self._grid.bus["vn_kv"][self.line_ex_to_subid].values.astype(dt_float)
+        self.storage_pu_to_kv = self._grid.bus["vn_kv"][self.storage_to_subid].values.astype(dt_float)
 
         self.thermal_limit_a = 1000 * np.concatenate((self._grid.line["max_i_ka"].values,
                                                       self._grid.trafo["sn_mva"].values / (np.sqrt(3) * self._grid.trafo["vn_hv_kv"].values)))
@@ -443,6 +516,9 @@ class PandaPowerBackend(Backend):
         self.prod_p = np.full(self.n_gen, dtype=dt_float, fill_value=np.NaN)
         self.prod_v = np.full(self.n_gen, dtype=dt_float, fill_value=np.NaN)
         self.prod_q = np.full(self.n_gen, dtype=dt_float, fill_value=np.NaN)
+        self.storage_p = np.full(self.n_storage, dtype=dt_float, fill_value=np.NaN)
+        self.storage_q = np.full(self.n_storage, dtype=dt_float, fill_value=np.NaN)
+        self.storage_v = np.full(self.n_storage, dtype=dt_float, fill_value=np.NaN)
         self._nb_bus_before = None
 
         # shunts data
@@ -489,14 +565,28 @@ class PandaPowerBackend(Backend):
             else:
                 self._big_topo_to_backend[pos_big_topo] = (l_id, l_id - self.__nb_powerline, 5)
 
+        self.theta_or = np.full(self.n_line, fill_value=np.NaN, dtype=dt_float)
+        self.theta_ex = np.full(self.n_line, fill_value=np.NaN, dtype=dt_float)
+        self.load_theta = np.full(self.n_load, fill_value=np.NaN, dtype=dt_float)
+        self.gen_theta = np.full(self.n_gen, fill_value=np.NaN, dtype=dt_float)
+        self.storage_theta = np.full(self.n_storage, fill_value=np.NaN, dtype=dt_float)
+
         self._topo_vect = self._get_topo_vect()
+        self.tol = 1e-5  # this is NOT the pandapower tolerance !!!! this is used to check if a storage unit
+        # produce / absorbs anything
+
         # Create a deep copy of itself in the initial state
         pp_backend_initial_state = copy.deepcopy(self)
         # Store it under super private attribute
         self.__pp_backend_initial_state = pp_backend_initial_state
 
+    def storage_deact_for_backward_comaptibility(self):
+        self._init_private_attrs()
+
     def _convert_id_topo(self, id_big_topo):
         """
+        INTERNAL
+
         .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
 
         convert an id of the big topo vector into:
@@ -509,6 +599,8 @@ class PandaPowerBackend(Backend):
 
     def apply_action(self, backendAction=None):
         """
+        INTERNAL
+
         .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
 
         Specific implementation of the method to apply an action modifying a powergrid in the pandapower format.
@@ -516,7 +608,7 @@ class PandaPowerBackend(Backend):
         if backendAction is None:
             return
 
-        active_bus, (prod_p, prod_v, load_p, load_q), topo__, shunts__ = backendAction()
+        active_bus, (prod_p, prod_v, load_p, load_q, storage), topo__, shunts__ = backendAction()
 
         tmp_prod_p = self._get_vector_inj["prod_p"](self._grid)
         if np.any(prod_p.changed):
@@ -538,6 +630,23 @@ class PandaPowerBackend(Backend):
         if np.any(load_q.changed):
             tmp_load_q.iloc[load_q.changed] = load_q.values[load_q.changed]
 
+        if self.n_storage > 0:
+            # active setpoint
+            tmp_stor_p = self._grid.storage["p_mw"]
+            if np.any(storage.changed):
+                tmp_stor_p.iloc[storage.changed] = storage.values[storage.changed]
+
+            # topology of the storage
+            stor_bus = backendAction.get_storages_bus()
+            new_bus_id = stor_bus.values[stor_bus.changed]  # id of the busbar 1 or 2 if
+            activated = new_bus_id > 0  # mask of storage that have been activated
+            new_bus_num = self.storage_to_subid[stor_bus.changed] + (new_bus_id - 1) * self.n_sub  # bus number
+            new_bus_num[~activated] = self.storage_to_subid[stor_bus.changed][~activated]
+            self._grid.storage["in_service"].values[stor_bus.changed] = activated
+            self._grid.storage["bus"].values[stor_bus.changed] = new_bus_num
+            self._topo_vect[self.storage_pos_topo_vect[stor_bus.changed]] = new_bus_num
+            self._topo_vect[self.storage_pos_topo_vect[stor_bus.changed][~activated]] = -1
+
         if self.shunts_data_available:
             shunt_p, shunt_q, shunt_bus = shunts__
 
@@ -558,7 +667,10 @@ class PandaPowerBackend(Backend):
         # i made at least a real change, so i implement it in the backend
         for id_el, new_bus in topo__:
             id_el_backend, id_topo, type_obj = self._big_topo_to_backend[id_el]
-            self._type_to_bus_set[type_obj](new_bus, id_el_backend, id_topo)
+
+            if type_obj is not None:
+                # storage unit are handled elsewhere
+                self._type_to_bus_set[type_obj](new_bus, id_el_backend, id_topo)
 
         bus_is = self._grid.bus["in_service"]
         for i, (bus1_status, bus2_status) in enumerate(active_bus):
@@ -648,6 +760,8 @@ class PandaPowerBackend(Backend):
 
     def runpf(self, is_dc=False):
         """
+        INTERNAL
+
         .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
 
         Run a power flow on the underlying _grid. This implements an optimization of the powerflow
@@ -662,6 +776,7 @@ class PandaPowerBackend(Backend):
                 # remove the warning if _grid non connex. And it that case load flow as not converged
                 warnings.filterwarnings("ignore", category=scipy.sparse.linalg.MatrixRankWarning)
                 warnings.filterwarnings("ignore", category=RuntimeWarning)
+                warnings.filterwarnings("ignore", category=DeprecationWarning)
                 if self._nb_bus_before is None:
                     self._pf_init = "dc"
                 elif nb_bus == self._nb_bus_before:
@@ -691,8 +806,8 @@ class PandaPowerBackend(Backend):
                     # sometimes pandapower does not detect divergence and put Nan.
                     raise pp.powerflow.LoadflowNotConverged("Isolated gen")
 
-                self.prod_p[:], self.prod_q[:], self.prod_v[:] = self._gens_info()
-                self.load_p[:], self.load_q[:], self.load_v[:] = self._loads_info()
+                self.prod_p[:], self.prod_q[:], self.prod_v[:], self.gen_theta[:] = self._gens_info()
+                self.load_p[:], self.load_q[:], self.load_v[:], self.load_theta[:] = self._loads_info()
                 if not is_dc:
                     if not np.all(np.isfinite(self.load_v)):
                         # TODO see if there is a better way here
@@ -718,6 +833,7 @@ class PandaPowerBackend(Backend):
                 self.q_or[:] = self._aux_get_line_info("q_from_mvar", "q_hv_mvar")
                 self.v_or[:] = self._aux_get_line_info("vm_from_pu", "vm_hv_pu")
                 self.a_or[:] = self._aux_get_line_info("i_from_ka", "i_hv_ka") * 1000
+                self.theta_or[:] = self._aux_get_line_info("va_from_degree", "va_hv_degree")
                 self.a_or[~np.isfinite(self.a_or)] = 0.
                 self.v_or[~np.isfinite(self.v_or)] = 0.
 
@@ -725,25 +841,38 @@ class PandaPowerBackend(Backend):
                 self.q_ex[:] = self._aux_get_line_info("q_to_mvar", "q_lv_mvar")
                 self.v_ex[:] = self._aux_get_line_info("vm_to_pu", "vm_lv_pu")
                 self.a_ex[:] = self._aux_get_line_info("i_to_ka", "i_lv_ka") * 1000
+                self.theta_ex[:] = self._aux_get_line_info("va_to_degree", "va_lv_degree")
                 self.a_ex[~np.isfinite(self.a_ex)] = 0.
                 self.v_ex[~np.isfinite(self.v_ex)] = 0.
 
                 # it seems that pandapower does not take into account disconencted powerline for their voltage
                 self.v_or[~self.line_status] = 0.
                 self.v_ex[~self.line_status] = 0.
-
                 self.v_or[:] *= self.lines_or_pu_to_kv
                 self.v_ex[:] *= self.lines_ex_pu_to_kv
 
                 self._nb_bus_before = None
                 self._grid._ppc["gen"][self._iref_slack, 1] = 0.
+
+                # handle storage units
+                # note that we have to look ourselves for disconnected storage
+                self.storage_p[:], self.storage_q[:], self.storage_v[:], self.storage_theta[:] = self._storages_info()
+                deact_storage = ~np.isfinite(self.storage_v)
+                if np.any(np.abs(self.storage_p[deact_storage]) > self.tol):
+                    raise pp.powerflow.LoadflowNotConverged("Isolated storage set to absorb / produce something")
+                self.storage_p[deact_storage] = 0.
+                self.storage_q[deact_storage] = 0.
+                self.storage_v[deact_storage] = 0.
+                self._grid.storage["in_service"].values[deact_storage] = False
+
                 self._topo_vect[:] = self._get_topo_vect()
-                return self._grid.converged
+                return self._grid.converged, None
 
         except pp.powerflow.LoadflowNotConverged as exc_:
             # of the powerflow has not converged, results are Nan
             self._reset_all_nan()
-            return False
+            msg = exc_.__str__()
+            return False, DivergingPowerFlow(f"powerflow diverged with error :\"{msg}\"")
 
     def _reset_all_nan(self):
         self.p_or[:] = np.NaN
@@ -760,10 +889,21 @@ class PandaPowerBackend(Backend):
         self.load_p[:] = np.NaN
         self.load_q[:] = np.NaN
         self.load_v[:] = np.NaN
+        self.storage_p[:] = np.NaN
+        self.storage_q[:] = np.NaN
+        self.storage_v[:] = np.NaN
         self._nb_bus_before = None
 
+        self.theta_or[:] = np.NaN
+        self.theta_ex[:] = np.NaN
+        self.load_theta[:] = np.NaN
+        self.gen_theta[:] = np.NaN
+        self.storage_theta[:] = np.NaN
+
     def copy(self):
         """
+        INTERNAL
+
         .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
 
         Performs a deep copy of the power :attr:`_grid`.
@@ -774,6 +914,8 @@ class PandaPowerBackend(Backend):
 
     def close(self):
         """
+        INTERNAL
+
         .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
 
         Called when the :class:`grid2op;Environment` has terminated, this function only reset the grid to a state
@@ -784,6 +926,8 @@ class PandaPowerBackend(Backend):
 
     def save_file(self, full_path):
         """
+        INTERNAL
+
         .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
 
         You might want to use it for debugging purpose only, and only if you develop yourself a backend.
@@ -796,6 +940,8 @@ class PandaPowerBackend(Backend):
 
     def get_line_status(self):
         """
+        INTERNAL
+
         .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
 
         As all the functions related to powerline, pandapower split them into multiple dataframe (some for transformers,
@@ -870,12 +1016,24 @@ class PandaPowerBackend(Backend):
             res[self.load_pos_topo_vect[i]] = 1 if bus_id == self.load_to_subid[i] else 2
             i += 1
 
+        if self.n_storage:
+            # storage can be deactivated by the environment for backward compatibility
+            i = 0
+            for bus_id in self._grid.storage["bus"].values:
+                status = self._grid.storage["in_service"].values[i]
+                if status:
+                    res[self.storage_pos_topo_vect[i]] = 1 if bus_id == self.storage_to_subid[i] else 2
+                else:
+                    res[self.storage_pos_topo_vect[i]] = -1
+                i += 1
+
         return res
 
     def _gens_info(self):
         prod_p = self.cst_1 * self._grid.res_gen["p_mw"].values.astype(dt_float)
         prod_q = self.cst_1 * self._grid.res_gen["q_mvar"].values.astype(dt_float)
         prod_v = self.cst_1 * self._grid.res_gen["vm_pu"].values.astype(dt_float) * self.prod_pu_to_kv
+        prod_theta = self.cst_1 * self._grid.res_gen["va_degree"].values.astype(dt_float)
         if self._iref_slack is not None:
             # slack bus and added generator are on same bus. I need to add power of slack bus to this one.
 
@@ -883,13 +1041,14 @@ class PandaPowerBackend(Backend):
             if "gen" in self._grid._ppc["internal"]:
                 prod_p[self._id_bus_added] += self._grid._ppc["internal"]["gen"][self._iref_slack, 1]
                 prod_q[self._id_bus_added] += self._grid._ppc["internal"]["gen"][self._iref_slack, 2]
-        return prod_p, prod_q, prod_v
+        return prod_p, prod_q, prod_v, prod_theta
 
     def _loads_info(self):
         load_p = self.cst_1 * self._grid.res_load["p_mw"].values.astype(dt_float)
         load_q = self.cst_1 * self._grid.res_load["q_mvar"].values.astype(dt_float)
         load_v = self._grid.res_bus.loc[self._grid.load["bus"].values]["vm_pu"].values.astype(dt_float) * self.load_pu_to_kv
-        return load_p, load_q, load_v
+        load_theta = self._grid.res_bus.loc[self._grid.load["bus"].values]["va_degree"].values.astype(dt_float)
+        return load_p, load_q, load_v, load_theta
 
     def generators_info(self):
         return self.cst_1 * self.prod_p, self.cst_1 * self.prod_q, self.cst_1 * self.prod_v
@@ -913,6 +1072,24 @@ class PandaPowerBackend(Backend):
         shunt_bus = shunt_bus.astype(dt_int)
         return shunt_p, shunt_q, shunt_v, shunt_bus
 
+    def storages_info(self):
+        return self.cst_1 * self.storage_p, self.cst_1 * self.storage_q, self.cst_1 * self.storage_v
+
+    def _storages_info(self):
+        if self.n_storage:
+            # this is because we support "backward comaptibility" feature. So the storage can be
+            # deactivated from the Environment...
+            p_storage = self._grid.res_storage["p_mw"].values.astype(dt_float)
+            q_storage = self._grid.res_storage["q_mvar"].values.astype(dt_float)
+            v_storage = self._grid.res_bus.loc[self._grid.storage["bus"].values]["vm_pu"].values.astype(dt_float) * self.storage_pu_to_kv
+            theta_storage = self._grid.res_bus.loc[self._grid.storage["bus"].values]["vm_pu"].values.astype(dt_float) * self.storage_pu_to_kv
+        else:
+            p_storage = np.zeros(shape=0, dtype=dt_float)
+            q_storage = np.zeros(shape=0, dtype=dt_float)
+            v_storage = np.zeros(shape=0, dtype=dt_float)
+            theta_storage = np.zeros(shape=0, dtype=dt_float)
+        return p_storage, q_storage, v_storage, theta_storage
+
     def sub_from_bus_id(self, bus_id):
         if bus_id >= self._number_true_line:
             return bus_id - self._number_true_line
