@@ -8,7 +8,7 @@
 
 import numpy as np
 import copy
-from grid2op.dtypes import dt_float
+from grid2op.dtypes import dt_float, dt_int
 from grid2op.Backend import Backend
 from grid2op.Exceptions import Grid2OpException
 
@@ -73,6 +73,7 @@ class BackendConverter(Backend):
         difcf = detailed_infos_for_cascading_failures
         self.source_backend = source_backend_class(detailed_infos_for_cascading_failures=difcf)
         self.target_backend = target_backend_class(detailed_infos_for_cascading_failures=difcf)
+        # if the target backend (the one performing the powerflows) needs a different file
         self.target_backend_grid_path = target_backend_grid_path
 
         # key: name in the source backend, value name in the target backend, for the substations
@@ -91,16 +92,22 @@ class BackendConverter(Backend):
         self._shunt_sr2tg = None
         self._topo_tg2sr = None
         self._topo_sr2tg = None
+        self._storage_tg2sr = None
+        self._storage_sr2tg = None
 
         # for redispatching data
         self.path_redisp = None
         self.name_redisp = None
         self.path_grid_layout = None
         self.name_grid_layout = None
+        self.path_storage_data = None
+        self.name_storage_data = None
 
         # for easier copy of np array
         self.cst1 = dt_float(1.)
 
+        # TODO storage check all this class ! + the doc of the backend
+
     def load_grid(self, path=None, filename=None):
         self.source_backend.load_grid(path, filename)
         # and now i load the target backend
@@ -111,26 +118,28 @@ class BackendConverter(Backend):
             self.target_backend.load_grid(path, filename)
 
     def _assert_same_grid(self):
-        """basic assertion that self and the target backend have the same grid"""
-        if self.n_sub != self.target_backend.n_sub:
+        """basic assertion that self and the target backend have the same grid
+        but not necessarily the same object at the same place of course"""
+        if type(self).n_sub != type(self.target_backend).n_sub:
             raise Grid2OpException(ERROR_NB_ELEMENTS.format("substations"))
-        if self.n_gen != self.target_backend.n_gen:
+        if type(self).n_gen != type(self.target_backend).n_gen:
             raise Grid2OpException(ERROR_NB_ELEMENTS.format("generators"))
-        if self.n_load != self.target_backend.n_load:
+        if type(self).n_load != type(self.target_backend).n_load:
             raise Grid2OpException(ERROR_NB_ELEMENTS.format("loads"))
-        if self.n_line != self.target_backend.n_line:
+        if type(self).n_line != type(self.target_backend).n_line:
             raise Grid2OpException(ERROR_NB_ELEMENTS.format("lines"))
+        if type(self).n_storage != type(self.target_backend).n_storage:
+            raise Grid2OpException(ERROR_NB_ELEMENTS.format("storages"))
 
     def _init_myself(self):
         # shortcut to set all information related to the class, except the name of the environment
         # this should been done when the source backend is fully initialized only
-        self.__class__ = self.init_grid(self.source_backend)
         self._assert_same_grid()
 
         # and now init all the converting vectors
         # a) for substation
-        self._sub_tg2sr = np.full(self.n_sub, fill_value=-1, dtype=np.int)
-        self._sub_sr2tg = np.full(self.n_sub, fill_value=-1, dtype=np.int)
+        self._sub_tg2sr = np.full(self.n_sub, fill_value=-1, dtype=dt_int)
+        self._sub_sr2tg = np.full(self.n_sub, fill_value=-1, dtype=dt_int)
         if self.sub_source_target is None:
             # automatic mode
             # I can only do it if the names matches
@@ -147,8 +156,8 @@ class BackendConverter(Backend):
                 self._sub_sr2tg[id_target] = id_source
 
         # b) for load
-        self._load_tg2sr = np.full(self.n_load, fill_value=-1, dtype=np.int)
-        self._load_sr2tg = np.full(self.n_load, fill_value=-1, dtype=np.int)
+        self._load_tg2sr = np.full(self.n_load, fill_value=-1, dtype=dt_int)
+        self._load_sr2tg = np.full(self.n_load, fill_value=-1, dtype=dt_int)
         # automatic mode
         self._auto_fill_vect_load_gen_shunt(n_element=self.n_load,
                                             source_2_id_sub=self.source_backend.load_to_subid,
@@ -158,8 +167,8 @@ class BackendConverter(Backend):
                                             nm="load")
 
         # c) for generator
-        self._gen_tg2sr = np.full(self.n_gen, fill_value=-1, dtype=np.int)
-        self._gen_sr2tg = np.full(self.n_gen, fill_value=-1, dtype=np.int)
+        self._gen_tg2sr = np.full(self.n_gen, fill_value=-1, dtype=dt_int)
+        self._gen_sr2tg = np.full(self.n_gen, fill_value=-1, dtype=dt_int)
         # automatic mode
         self._auto_fill_vect_load_gen_shunt(n_element=self.n_gen,
                                             source_2_id_sub=self.source_backend.gen_to_subid,
@@ -169,21 +178,32 @@ class BackendConverter(Backend):
                                             nm="gen")
 
         # d) for powerline
-        self._line_tg2sr = np.full(self.n_line, fill_value=-1, dtype=np.int)
-        self._line_sr2tg = np.full(self.n_line, fill_value=-1, dtype=np.int)
+        self._line_tg2sr = np.full(self.n_line, fill_value=-1, dtype=dt_int)
+        self._line_sr2tg = np.full(self.n_line, fill_value=-1, dtype=dt_int)
         # automatic
         self._auto_fill_vect_powerline()
 
         # e) and now the topology vectors.
-        self._topo_tg2sr = np.full(self.dim_topo, fill_value=-1, dtype=np.int)
-        self._topo_sr2tg = np.full(self.dim_topo, fill_value=-1, dtype=np.int)
+        self._topo_tg2sr = np.full(self.dim_topo, fill_value=-1, dtype=dt_int)
+        self._topo_sr2tg = np.full(self.dim_topo, fill_value=-1, dtype=dt_int)
         self._auto_fill_vect_topo()
 
+        # f) for the storage units
+        self._storage_tg2sr = np.full(self.n_storage, fill_value=-1, dtype=dt_int)
+        self._storage_sr2tg = np.full(self.n_storage, fill_value=-1, dtype=dt_int)
+        # automatic mode
+        self._auto_fill_vect_load_gen_shunt(n_element=self.n_storage,
+                                            source_2_id_sub=self.source_backend.storage_to_subid,
+                                            target_2_id_sub=self.target_backend.storage_to_subid,
+                                            tg2sr=self._storage_tg2sr,
+                                            sr2tg=self._storage_sr2tg,
+                                            nm="storage")
+
         # shunt are available if both source and target provide it
         self.shunts_data_available = self.source_backend.shunts_data_available and self.target_backend.shunts_data_available
         if self.shunts_data_available:
-            self._shunt_tg2sr = np.full(self.n_shunt, fill_value=-1, dtype=np.int)
-            self._shunt_sr2tg = np.full(self.n_shunt, fill_value=-1, dtype=np.int)
+            self._shunt_tg2sr = np.full(self.n_shunt, fill_value=-1, dtype=dt_int)
+            self._shunt_sr2tg = np.full(self.n_shunt, fill_value=-1, dtype=dt_int)
             # automatic mode
             self._auto_fill_vect_load_gen_shunt(n_element=self.n_shunt,
                                                 source_2_id_sub=self.source_backend.shunt_to_subid,
@@ -193,13 +213,6 @@ class BackendConverter(Backend):
                                                 nm="shunt")
         self.set_thermal_limit(self.target_backend.thermal_limit_a[self._line_tg2sr])
 
-        if self.path_redisp is not None:
-            # redispatching data were available
-            super().load_redispacthing_data(self.path_redisp, name=self.name_redisp)
-        if self.path_grid_layout is not None:
-            # grid layout data were available
-            super().load_grid_layout(self.path_grid_layout, self.name_grid_layout)
-
     def _get_possible_target_ids(self, id_source, source_2_id_sub, target_2_id_sub, nm):
         id_sub_source = source_2_id_sub[id_source]
         id_sub_target = self._sub_tg2sr[id_sub_source]
@@ -208,10 +221,16 @@ class BackendConverter(Backend):
             raise RuntimeError(ERROR_ELEMENT_CONNECTED.format(nm, id_sub_target, id_sub_source))
         return id_sub_target, ids_target
 
-    def _auto_fill_vect_load_gen_shunt(self, n_element, source_2_id_sub, target_2_id_sub,
+    def _auto_fill_vect_load_gen_shunt(self,
+                                       n_element,
+                                       source_2_id_sub, target_2_id_sub,
                                        tg2sr, sr2tg,
                                        nm):
-        nb_load_per_sub = np.zeros(self.n_sub, dtype=np.int)
+        nb_load_per_sub = np.zeros(self.n_sub, dtype=dt_int)
+        if source_2_id_sub.shape[0] != n_element:
+            raise RuntimeError("Impossible to convert backend that do not have the same number of objects")
+        if target_2_id_sub.shape[0] != n_element:
+            raise RuntimeError("Impossible to convert backend that do not have the same number of objects")
         for id_source in range(n_element):
             id_sub_target, id_target = self._get_possible_target_ids(id_source, source_2_id_sub, target_2_id_sub, nm)
             id_target = id_target[nb_load_per_sub[id_sub_target]]
@@ -223,7 +242,7 @@ class BackendConverter(Backend):
 
     def _auto_fill_vect_powerline(self):
         # automatic matching
-        nb_load_per_sub = np.zeros((self.n_sub, self.n_sub), dtype=np.int)
+        nb_load_per_sub = np.zeros((self.n_sub, self.n_sub), dtype=dt_int)
         n_element = self.n_line
         source_or_2_id_sub = self.source_backend.line_or_to_subid
         target_or_2_id_sub = self.target_backend.line_or_to_subid
@@ -266,6 +285,10 @@ class BackendConverter(Backend):
                                       self.source_backend.line_ex_pos_topo_vect,
                                       self.target_backend.line_ex_pos_topo_vect,
                                       self._line_sr2tg)
+        self._auto_fill_vect_topo_aux(self.n_storage,
+                                      self.source_backend.storage_pos_topo_vect,
+                                      self.target_backend.storage_pos_topo_vect,
+                                      self._storage_sr2tg)
 
     def _auto_fill_vect_topo_aux(self, n_elem, source_pos, target_pos, sr2tg):
         # TODO that might not be working as intented... it always says it's the identity...
@@ -274,22 +297,37 @@ class BackendConverter(Backend):
 
     def assert_grid_correct(self):
         # this is done before a call to this function, by the environment
-        self.source_backend.set_env_name(self.env_name)
-        self.target_backend.set_env_name(self.env_name)
+        env_name = type(self).env_name
+        type(self.target_backend).set_env_name(env_name)
+        type(self.source_backend).set_env_name(env_name)
+        self._init_class_attr(obj=self.source_backend)
+
+        if self.path_redisp is not None:
+            # redispatching data were available
+            super().load_redispacthing_data(self.path_redisp, name=self.name_redisp)
+            self.source_backend.load_redispacthing_data(self.path_redisp, name=self.name_redisp)
+            self.target_backend.load_redispacthing_data(self.path_redisp, name=self.name_redisp)
+        if self.path_storage_data is not None:
+            super().load_storage_data(self.path_storage_data, self.name_storage_data)
+            self.source_backend.load_storage_data(self.path_redisp, name=self.name_redisp)
+            self.target_backend.load_storage_data(self.path_redisp, name=self.name_redisp)
+        if self.path_grid_layout is not None:
+            # grid layout data were available
+            super().load_grid_layout(self.path_grid_layout, self.name_grid_layout)
+            self.source_backend.load_grid_layout(self.path_redisp, name=self.name_redisp)
+
+        # init the target backend (the one that does the computation and that is initialized)
+        self.target_backend.assert_grid_correct()
+        # initialize the other one, because, well the grid should be seen from both backend
+        self.source_backend._init_class_attr(obj=self)
+        self.source_backend.assert_grid_correct()
+
+        # and this should be called after all the rest
+        super().assert_grid_correct()
 
         # everything went well, so i can properly terminate my initialization
         self._init_myself()
 
-        # the next is not done as it is supposed to be done in "assert_grid_correct_after_powerflow"
-        self.source_backend.__class__ = self.source_backend.init_grid(self)
-        self.target_backend.__class__ = self.target_backend.init_grid(self)  # for this one i am not sure
-
-        # now i assert that the powergrids are ok
-        self.source_backend.assert_grid_correct()
-        self.target_backend.assert_grid_correct()
-
-        # and this should be called after all the rest
-        super().assert_grid_correct()
         if self.sub_source_target is None:
             # automatic mode for substations, names must match
             assert np.all(self.target_backend.name_sub[self._sub_tg2sr] == self.source_backend.name_sub)
@@ -304,6 +342,9 @@ class BackendConverter(Backend):
         if self.shunts_data_available:
             self._check_both_consistent(self._shunt_tg2sr, self._shunt_sr2tg)
 
+        # finally check that powergrids are identical (up to the env name)
+        type(self.target_backend).same_grid_class(type(self.source_backend))
+
     def _check_vect_valid(self, vect):
         assert np.all(vect >= 0), "invalid vector: some element are not found in either source or target"
         assert sorted(np.unique(vect)) == sorted(vect), "invalid vector: some element are not found in either source or target"
@@ -427,7 +468,8 @@ class BackendConverter(Backend):
         target_action.reorder(no_load=self._load_sr2tg,
                               no_gen=self._gen_sr2tg,
                               no_topo=self._topo_sr2tg,
-                              no_shunt=self._shunt_sr2tg)
+                              no_shunt=self._shunt_sr2tg,
+                              no_storage=self._storage_sr2tg)
         return target_action
 
     def load_redispacthing_data(self, path, name='prods_charac.csv'):
@@ -435,6 +477,11 @@ class BackendConverter(Backend):
         self.path_redisp = path
         self.name_redisp = name
 
+    def load_storage_data(self, path, name='storage_units_charac.csv'):
+        # data are loaded with the name of the source backend, i need to map it to the target backend too
+        self.path_storage_data = path
+        self.name_storage_data = name
+
     def load_grid_layout(self, path, name='grid_layout.json'):
         self.path_grid_layout = path
         self.name_grid_layout = name
