@@ -10,6 +10,7 @@ import os
 import importlib.util
 import numpy as np
 import json
+import warnings
 
 from grid2op.Environment import Environment
 from grid2op.Backend import Backend, PandaPowerBackend
@@ -30,24 +31,28 @@ DIFFICULTY_NAME = "difficulty"
 CHALLENGE_NAME = "competition"
 ERR_MSG_KWARGS = {
     "backend": "The backend of the environment (keyword \"backend\") must be an instance of grid2op.Backend",
-    "observation_class": "The type of observation of the environment (keyword \"observation_class\")" \
+    "observation_class": "The type of observation of the environment (keyword \"observation_class\")"
     " must be a subclass of grid2op.BaseObservation",
     "param": "The parameters of the environment (keyword \"param\") must be an instance of grid2op.Parameters",
-    "gamerules_class": "The type of rules of the environment (keyword \"gamerules_class\")" \
+    "gamerules_class": "The type of rules of the environment (keyword \"gamerules_class\")"
     " must be a subclass of grid2op.BaseRules",
-    "reward_class": "The type of reward in the environment (keyword \"reward_class\") must be a subclass of grid2op.BaseReward",
-    "action_class": "The type of action of the environment (keyword \"action_class\") must be a subclass of grid2op.BaseAction",
-    "data_feeding_kwargs": "The argument to build the data generation process [chronics]" \
+    "reward_class": "The type of reward in the environment (keyword \"reward_class\") must be a subclass of "
+                    "grid2op.BaseReward",
+    "action_class": "The type of action of the environment (keyword \"action_class\") must be a subclass of "
+                    "grid2op.BaseAction",
+    "data_feeding_kwargs": "The argument to build the data generation process [chronics]"
     "  (keyword \"data_feeding_kwargs\") should be a dictionnary.",
-    "chronics_class": "The argument to build the data generation process [chronics] (keyword \"chronics_class\")" \
+    "chronics_class": "The argument to build the data generation process [chronics] (keyword \"chronics_class\")"
     " should be a class that inherit grid2op.Chronics.GridValue.",
-    "chronics_handler": "The argument to build the data generation process [chronics] (keyword \"data_feeding\")" \
+    "chronics_handler": "The argument to build the data generation process [chronics] (keyword \"data_feeding\")"
     " should be a class that inherit grid2op.ChronicsHandler.ChronicsHandler.",
-    "voltagecontroler_class": "The argument to build the online controler for chronics (keyword \"volagecontroler_class\")" \
+    "voltagecontroler_class": "The argument to build the online controler for chronics (keyword "
+                              "\"volagecontroler_class\")"
     " should be a class that inherit grid2op.VoltageControler.ControlVoltageFromFile.",
-    "names_chronics_to_grid": "The converter between names (keyword \"names_chronics_to_backend\") should be a dictionnary.",
+    "names_chronics_to_grid": "The converter between names (keyword \"names_chronics_to_backend\") "
+                              "should be a dictionnary.",
     "other_rewards": "The argument to build the online controler for chronics (keyword \"other_rewards\") "
-                     "should be dictionnary.",
+                     "should be dictionary.",
 
     "chronics_path": "The path where the data is located (keyword \"chronics_path\") should be a string.",
     "grid_path": "The path where the grid is located (keyword \"grid_path\") should be a string.",
@@ -61,8 +66,8 @@ ERR_MSG_KWARGS = {
     "opponent_budget_class": "The opponent budget class (\"opponent_budget_class\") should derive from "
                              "\"BaseActionBudget\".",
     "opponent_budget_per_ts": "The increase of the opponent's budget (\"opponent_budget_per_ts\") should be a float.",
-    "kwargs_opponent": "The extra kwargs argument used to properly initiliazed the opponent "
-                       "(\"kwargs_opponent\") shoud "
+    "kwargs_opponent": "The extra kwargs argument used to properly initialized the opponent "
+                       "(\"kwargs_opponent\") should "
                        "be a dictionary.",
     DIFFICULTY_NAME: "Unknown difficulty level {difficulty} for this environment. Authorized difficulties are "
                      "{difficulties}"
@@ -76,8 +81,8 @@ NAME_CONFIG_FILE = "config.py"
 
 def _check_kwargs(kwargs):
     for el in kwargs:
-        if not el in ERR_MSG_KWARGS.keys():
-            raise EnvError("Unknown keyword argument \"{}\" used to create an Environement. "
+        if el not in ERR_MSG_KWARGS.keys():
+            raise EnvError("Unknown keyword argument \"{}\" used to create an Environment. "
                            "No Environment will be created. "
                            "Accepted keyword arguments are {}".format(el, ERR_MSG_KWARGS.keys()))
 
@@ -87,12 +92,18 @@ def _check_path(path, info):
         raise EnvError("Cannot find {}. {}".format(path, info))
 
 
-def make_from_dataset_path(dataset_path="/", _add_to_name="", **kwargs):
+def make_from_dataset_path(dataset_path="/",
+                           _add_to_name="",
+                           _compat_glop_version=None,
+                           **kwargs):
     """
+    INTERNAL USE ONLY
+
     .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
 
         Prefer using the :func:`grid2op.make` function.
 
+
     This function is a shortcut to rapidly create environments within the grid2op Framework. We don't
     recommend using directly this function. Prefer using the :func:`make` function.
 
@@ -187,6 +198,9 @@ def make_from_dataset_path(dataset_path="/", _add_to_name="", **kwargs):
     _add_to_name:
         Internal, used for test only. Do not attempt to modify under any circumstances.
 
+    _compat_glop_version:
+        Internal, used for test only. Do not attempt to modify under any circumstances.
+
     Returns
     -------
     env: :class:`grid2op.Environment.Environment`
@@ -210,7 +224,11 @@ def make_from_dataset_path(dataset_path="/", _add_to_name="", **kwargs):
     else:
         # otherwise use it
         chronics_path_abs = os.path.abspath(chronics_path)
-    _check_path(chronics_path_abs, "Dataset chronics folder")
+    exc_chronics = None
+    try:
+        _check_path(chronics_path_abs, "Dataset chronics folder")
+    except Exception as exc_:
+        exc_chronics = exc_
 
     # Compute and find backend/grid file
     grid_path = _get_default_aux("grid_path", kwargs,
@@ -224,7 +242,11 @@ def make_from_dataset_path(dataset_path="/", _add_to_name="", **kwargs):
 
     # Compute and find grid layout file
     grid_layout_path_abs = os.path.abspath(os.path.join(dataset_path_abs, NAME_GRID_LAYOUT_FILE))
-    _check_path(grid_layout_path_abs, "Dataset grid layout")
+    try:
+        _check_path(grid_layout_path_abs, "Dataset grid layout")
+    except EnvError as exc_:
+        warnings.warn(f"Impossible to load the coordinate of the substation with error: \"{exc_}\". Expect some issue "
+                      f"if you attempt to plot the grid.")
 
     # Check provided config overrides are valid
     _check_kwargs(kwargs)
@@ -239,16 +261,18 @@ def make_from_dataset_path(dataset_path="/", _add_to_name="", **kwargs):
         config_module = importlib.util.module_from_spec(spec)
         spec.loader.exec_module(config_module)
         config_data = config_module.config
-    except Exception as e:
-        print (e)
+    except Exception as exc_:
+        print(exc_)
         raise EnvError("Invalid dataset config file: {}".format(config_path_abs)) from None
 
     # Get graph layout
+    graph_layout = None
     try:
         with open(grid_layout_path_abs) as layout_fp:
             graph_layout = json.load(layout_fp)
-    except Exception as e:
-        raise EnvError("Dataset {} doesn't have a valid graph layout".format(config_path_abs))
+    except Exception as exc_:
+        warnings.warn("Dataset {} doesn't have a valid graph layout. Expect some failures when attempting "
+                      "to plot the grid. Error was: {}".format(config_path_abs, exc_))
 
     # Get thermal limits
     thermal_limits = None
@@ -298,9 +322,10 @@ def make_from_dataset_path(dataset_path="/", _add_to_name="", **kwargs):
                 my_difficulty = kwargs[DIFFICULTY_NAME]
                 try:
                     my_difficulty = str(my_difficulty)
-                except:
+                except Exception as exc_:
                     raise EnvError("Impossible to convert your difficulty into a valid string. Please make sure to "
-                                   "pass a string (eg \"2\") and not something else (eg. int(2)) as a difficulty")
+                                   "pass a string (eg \"2\") and not something else (eg. int(2)) as a difficulty."
+                                   "Error was \n{}".format(exc_))
                 if my_difficulty in dict_:
                     param.init_from_dict(dict_[my_difficulty])
                 else:
@@ -386,7 +411,7 @@ def make_from_dataset_path(dataset_path="/", _add_to_name="", **kwargs):
                                            defaultinstance=default_chronics_kwargs,
                                            msg_error=ERR_MSG_KWARGS["data_feeding_kwargs"])
     for el in default_chronics_kwargs:
-        if not el in data_feeding_kwargs:
+        if el not in data_feeding_kwargs:
             data_feeding_kwargs[el] = default_chronics_kwargs[el]
 
     ### the chronics generator
@@ -395,6 +420,10 @@ def make_from_dataset_path(dataset_path="/", _add_to_name="", **kwargs):
                                            defaultClass=data_feeding_kwargs["chronicsClass"],
                                            msg_error=ERR_MSG_KWARGS["chronics_class"],
                                            isclass=True)
+    if (chronics_class_used != ChangeNothing) and exc_chronics is not None:
+        raise EnvError(f"Impossible to find the chronics for your environment. Please make sure to provide "
+                       f"a folder \"{NAME_CHRONICS_FOLDER}\" within your environment folder.")
+
     data_feeding_kwargs["chronicsClass"] = chronics_class_used
     data_feeding = _get_default_aux("data_feeding", kwargs,
                                     defaultClassApp=ChronicsHandler,
@@ -499,6 +528,7 @@ def make_from_dataset_path(dataset_path="/", _add_to_name="", **kwargs):
                       opponent_budget_per_ts=opponent_budget_per_ts,
                       opponent_budget_class=opponent_budget_class,
                       kwargs_opponent=kwargs_opponent,
+                      _compat_glop_version=_compat_glop_version
                       )
 
     # Update the thermal limit if any
