@@ -17,6 +17,8 @@ from grid2op.Exceptions import Grid2OpException
 
 class _ObsCH(ChangeNothing):
     """
+    INTERNAL
+
     .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
 
     This class is reserved to internal use. Do not attempt to do anything with it.
@@ -27,6 +29,8 @@ class _ObsCH(ChangeNothing):
 
 class _ObsEnv(BaseEnv):
     """
+    INTERNAL
+
     .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
 
     This class is an 'Emulator' of a :class:`grid2op.Environment.Environment` used to be able to 'simulate'
@@ -40,14 +44,12 @@ class _ObsEnv(BaseEnv):
     """
     def __init__(self,
                  backend_instanciated,
-                 completeActionClass,
                  parameters,
                  reward_helper,
                  obsClass,
                  action_helper,
                  thermal_limit_a,
                  legalActClass,
-                 donothing_act,
                  helper_action_class,
                  helper_action_env,
                  other_rewards={}):
@@ -58,6 +60,7 @@ class _ObsEnv(BaseEnv):
         self._helper_action_class = helper_action_class
         self._reward_helper = reward_helper
         self._obsClass = None
+        self._obsClass = obsClass.init_grid(type(backend_instanciated))
 
         self.gen_activeprod_t_init = np.zeros(self.n_gen, dtype=dt_float)
         self.gen_activeprod_t_redisp_init = np.zeros(self.n_gen, dtype=dt_float)
@@ -73,7 +76,7 @@ class _ObsEnv(BaseEnv):
                            backend=backend_instanciated,
                            names_chronics_to_backend=None,
                            actionClass=action_helper.actionClass,
-                           observationClass=obsClass,
+                           observationClass=self._obsClass,
                            rewardClass=None,
                            legalActClass=legalActClass)
         self.no_overflow_disconnection = parameters.NO_OVERFLOW_DISCONNECTION
@@ -94,6 +97,32 @@ class _ObsEnv(BaseEnv):
         self.opp_space_state = None
         self.opp_state = None
 
+        # storage
+        self._storage_current_charge_init = None
+        self._storage_previous_charge_init = None
+        self._action_storage_init = None
+        self._amount_storage_init = None
+        self._amount_storage_prev_init = None
+        self._storage_power_init = None
+
+        # storage unit
+        self._storage_current_charge_init = np.zeros(self.n_storage, dtype=dt_float)
+        self._storage_previous_charge_init = np.zeros(self.n_storage, dtype=dt_float)
+        self._action_storage_init = np.zeros(self.n_storage, dtype=dt_float)
+        self._storage_power_init = np.zeros(self.n_storage, dtype=dt_float)
+        self._amount_storage_init = 0.
+        self._amount_storage_prev_init = 0.
+
+        # curtailment
+        self._limit_curtailment_init = np.zeros(self.n_gen, dtype=dt_float)
+        self._gen_before_curtailment_init = np.zeros(self.n_gen, dtype=dt_float)
+        self._sum_curtailment_mw_init = 0.
+        self._sum_curtailment_mw_prev_init = 0.
+
+    def _init_myclass(self):
+        """this class has already all the powergrid information: it is initialized in the obs space !"""
+        pass
+
     def _init_backend(self,
                       init_grid_path,
                       chronics_handler,
@@ -101,12 +130,12 @@ class _ObsEnv(BaseEnv):
                       names_chronics_to_backend,
                       actionClass,
                       observationClass,
-                      rewardClass, legalActClass):
+                      rewardClass,
+                      legalActClass):
         self._env_dc = self.parameters.ENV_DC
         self.chronics_handler = chronics_handler
         self.backend = backend
-        self._has_been_initialized()
-        self._obsClass = observationClass
+        self._has_been_initialized()  # really important to include this piece of code! and just here after the
 
         if not issubclass(legalActClass, BaseRules):
             raise Grid2OpException(
@@ -115,7 +144,7 @@ class _ObsEnv(BaseEnv):
                     type(legalActClass)))
         self._game_rules = RulesChecker(legalActClass=legalActClass)
         self._legalActClass = legalActClass
-        self._helper_action_player = self._do_nothing
+        self._action_space = self._do_nothing
         self.backend.set_thermal_limit(self._thermal_limit_a)
         self._create_opponent()
 
@@ -127,13 +156,20 @@ class _ObsEnv(BaseEnv):
         # backend has loaded everything
         self._line_status = np.ones(shape=self.n_line, dtype=dt_bool)
         self._hazard_duration = np.zeros(shape=self.n_line, dtype=dt_int)
-        self._has_been_initialized()
 
     def _do_nothing(self, x):
+        """
+        this is should be only called within _Obsenv.step, and there, only return the "do nothing"
+        action.
+
+        This is why this function is used as the "obsenv action space"
+        """
         return self._do_nothing_act
 
     def _update_actions(self):
         """
+        INTERNAL
+
         .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
 
         Retrieve the actions to perform the update of the underlying powergrid represented by
@@ -152,6 +188,8 @@ class _ObsEnv(BaseEnv):
 
     def copy(self):
         """
+        INTERNAL
+
         .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
 
         Implement the deep copy of this instance.
@@ -170,6 +208,8 @@ class _ObsEnv(BaseEnv):
 
     def init(self, new_state_action, time_stamp, timestep_overflow, topo_vect, time_step=1):
         """
+        INTERNAL
+
         .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
 
         Initialize a "forecasted grid state" based on the new injections, possibly new topological modifications etc.
@@ -219,32 +259,58 @@ class _ObsEnv(BaseEnv):
                                                                            "load_q": self._load_q}
                                                              })
         self._backend_action_set += new_state_action
+        # for storage unit
+        self._backend_action_set.storage_power.values[:] = 0.
+
+        # for curtailment
+        if self._env_modification is not None:
+            self._env_modification._dict_inj = {}
+
         self.is_init = True
         self.current_obs.reset()
         self.time_stamp = time_stamp
         self._timestep_overflow[:] = timestep_overflow
 
+    def _get_new_prod_setpoint(self, action):
+        new_p = 1. * self._backend_action_set.prod_p.values
+        if "prod_p" in action._dict_inj:
+            tmp = action._dict_inj["prod_p"]
+            indx_ok = np.isfinite(tmp)
+            new_p[indx_ok] = tmp[indx_ok]
+
+        # modification of the environment always override the modification of the agents (if any)
+        # TODO have a flag there if this is the case.
+        if "prod_p" in self._env_modification._dict_inj:
+            # modification of the production setpoint value
+            tmp = self._env_modification._dict_inj["prod_p"]
+            indx_ok = np.isfinite(tmp)
+            new_p[indx_ok] = tmp[indx_ok]
+        return new_p
+
     def _update_vector_with_timestep(self, time_step):
         """
+        INTERNAL
+
         .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
 
         update the value of the "time dependant" attributes
         """
         self._times_before_line_status_actionable[:] = np.maximum(self._times_before_line_status_actionable - time_step,
-                                                                  0.)
+                                                                  0)
         self._times_before_topology_actionable[:] = np.maximum(self._times_before_topology_actionable - time_step,
-                                                               0.)
+                                                               0)
 
         still_in_maintenance = (self._duration_next_maintenance > time_step) & (self._time_next_maintenance == 0)
-        reconnected = (self._duration_next_maintenance < time_step) & (self._time_next_maintenance == 0)
+        reconnected = (self._duration_next_maintenance <= time_step) & (self._time_next_maintenance == 0)
         first_ts_maintenance = self._time_next_maintenance == time_step
 
         # powerline that are still in maintenance at this time step
         self._time_next_maintenance[still_in_maintenance] = 0
-        self._duration_next_maintenance[still_in_maintenance] -= 1
+        self._duration_next_maintenance[still_in_maintenance] -= time_step
 
         # powerline that will be in maintenance at this time step
         self._time_next_maintenance[first_ts_maintenance] = 0
+        self._duration_next_maintenance[first_ts_maintenance] -= time_step
 
         # powerline that won't be in maintenance at this time step
         self._time_next_maintenance[reconnected] = -1
@@ -267,6 +333,8 @@ class _ObsEnv(BaseEnv):
 
     def _reset_to_orig_state(self):
         """
+        INTERNAL
+
         .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
 
         reset this "environment" to the state it should be
@@ -277,8 +345,24 @@ class _ObsEnv(BaseEnv):
         self._backend_action = copy.deepcopy(self._backend_action_set)
         self._oppSpace._set_state(self.opp_space_state, self.opp_state)
 
+        # storage unit
+        self._storage_current_charge[:] = self._storage_current_charge_init
+        self._storage_previous_charge[:] = self._storage_previous_charge_init
+        self._action_storage[:] = self._action_storage_init
+        self._storage_power[:] = self._storage_power_init
+        self._amount_storage = self._amount_storage_init
+        self._amount_storage_prev = self._amount_storage_prev_init
+
+        # curtailment
+        self._limit_curtailment[:] = self._limit_curtailment_init
+        self._gen_before_curtailment[:] = self._gen_before_curtailment_init
+        self._sum_curtailment_mw = self._sum_curtailment_mw_init
+        self._sum_curtailment_mw_prev = self._sum_curtailment_mw_prev_init
+
     def simulate(self, action):
         """
+        INTERNAL
+
         .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
             Prefer using `obs.simulate(action)`
 
@@ -324,6 +408,8 @@ class _ObsEnv(BaseEnv):
 
     def get_obs(self):
         """
+        INTERNAL
+
         .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
 
         Method to retrieve the "forecasted grid" as a valid observation object.
@@ -339,6 +425,8 @@ class _ObsEnv(BaseEnv):
 
     def update_grid(self, env):
         """
+        INTERNAL
+
         .. warning:: /!\\\\ Internal, do not use unless you know what you are doing /!\\\\
 
         Update this "emulated" environment with the real powergrid.
@@ -352,7 +440,6 @@ class _ObsEnv(BaseEnv):
             A reference to the environment
         """
         real_backend = env.backend
-        self._reward_helper = env._reward_helper
 
         self._load_p, self._load_q, self._load_v = real_backend.loads_info()
         self._prod_p, self._prod_q, self._prod_v = real_backend.generators_info()
@@ -379,11 +466,25 @@ class _ObsEnv(BaseEnv):
         self.target_dispatch_init[:] = env._target_dispatch
         self.actual_dispatch_init[:] = env._actual_dispatch
         self.opp_space_state, self.opp_state = env._oppSpace._get_state()
-        # TODO check redispatching and simulate are working as intended
-        # TODO also update the status of hazards, maintenance etc.
-        # TODO and simulate also when a maintenance is forcasted!
-        # TODO add the opponent budget here (should decrease with the time step :scared:) -> we really need to address
-        # all that before 1.0.0
+
+        # storage units
+        # TODO this is not time independant... i set up the previous charge of the obs env to be
+        # set current charge of the simulated env on purpose
+        self._storage_current_charge_init[:] = env._storage_current_charge
+        self._storage_previous_charge_init[:] = env._storage_previous_charge
+        self._action_storage_init[:] = env._action_storage
+        self._amount_storage_init = env._amount_storage
+        self._amount_storage_prev_init = env._amount_storage_prev
+        self._storage_power_init[:] = env._storage_power
+
+        # curtailment
+        self._limit_curtailment_init[:] = env._limit_curtailment
+        self._gen_before_curtailment_init[:] = env._gen_before_curtailment
+        self._sum_curtailment_mw_init = env._sum_curtailment_mw
+        self._sum_curtailment_mw_prev_init = env._sum_curtailment_mw_prev
+
+        # time delta
+        self.delta_time_seconds = env.delta_time_seconds
 
     def get_current_line_status(self):
         return self._line_status == 1
\ No newline at end of file
