@@ -1,9 +1,10 @@
 __all__ = ["OpponentSpace", "BaseActionBudget", "BaseOpponent", "UnlimitedBudget",
-           "RandomLineOpponent", "NeverAttackBudget"]
+           "RandomLineOpponent", "WeightedRandomOpponent", "NeverAttackBudget"]
 
 from grid2op.Opponent.OpponentSpace import OpponentSpace
 from grid2op.Opponent.BaseActionBudget import BaseActionBudget
 from grid2op.Opponent.BaseOpponent import BaseOpponent
 from grid2op.Opponent.UnlimitedBudget import UnlimitedBudget
 from grid2op.Opponent.RandomLineOpponent import RandomLineOpponent
+from grid2op.Opponent.WeightedRandomOpponent import WeightedRandomOpponent
 from grid2op.Opponent.NeverAttackBudget import NeverAttackBudget
