@@ -5,18 +5,18 @@
 # you can obtain one at http://mozilla.org/MPL/2.0/.
 # SPDX-License-Identifier: MPL-2.0
 # This file is part of Grid2Op, Grid2Op a testbed platform to model sequential decision making in power systems.
+
 import numpy as np
 from gym import spaces
 
-
+from grid2op.Environment import Environment, MultiMixEnvironment, BaseMultiProcessEnvironment
 from grid2op.gym_compat.gym_space_converter import _BaseGymSpaceConverter
 from grid2op.Observation import BaseObservation
-from grid2op.dtypes import dt_int, dt_bool
+from grid2op.dtypes import dt_int, dt_bool, dt_float
 from grid2op.gym_compat.base_gym_attr_converter import BaseGymAttrConverter
 
 
 class GymObservationSpace(_BaseGymSpaceConverter):
-    # deals with the observation space (rather easy)
     """
     This class allows to transform the observation space into a gym space.
 
@@ -50,6 +50,9 @@ class GymObservationSpace(_BaseGymSpaceConverter):
 
     """
     def __init__(self, env, dict_variables=None):
+        if not isinstance(env, (Environment, MultiMixEnvironment, BaseMultiProcessEnvironment)):
+            raise RuntimeError("GymActionSpace must be created with an Environment of an ActionSpace (or a Converter)")
+
         self._init_env = env
         self.initial_obs_space = self._init_env.observation_space
         dict_ = {}  # will represent the gym.Dict space
@@ -99,6 +102,14 @@ class GymObservationSpace(_BaseGymSpaceConverter):
         my_dict = self.get_dict_encoding()
         if fun is not None and not isinstance(fun, BaseGymAttrConverter):
             raise RuntimeError("Impossible to initialize a converter with a function of type {}".format(type(fun)))
+
+        if fun is not None and not fun.is_init_space():
+            if key in my_dict:
+                fun.initialize_space(my_dict[key])
+            elif key in self.spaces:
+                fun.initialize_space(self.spaces[key])
+            else:
+                raise RuntimeError(f"Impossible to find key {key} in your observation space")
         my_dict[key] = fun
         res = GymObservationSpace(self._init_env, my_dict)
         return res
@@ -108,6 +119,8 @@ class GymObservationSpace(_BaseGymSpaceConverter):
         for attr_nm, sh, dt in zip(observation_space.attr_list_vect,
                                    observation_space.shape,
                                    observation_space.dtype):
+            if sh == 0:
+                continue
             my_type = None
             shape = (sh,)
             if attr_nm in dict_variables:
@@ -137,7 +150,7 @@ class GymObservationSpace(_BaseGymSpaceConverter):
                     my_type = spaces.Box(low=0,
                                          high=max(env_params.NB_TIMESTEP_COOLDOWN_LINE,
                                                   env_params.NB_TIMESTEP_RECONNECTION,
-                                                  opponent_space.attack_cooldown
+                                                  opponent_space.attack_duration
                                                   ),
                                          shape=shape,
                                          dtype=dt)
@@ -159,11 +172,11 @@ class GymObservationSpace(_BaseGymSpaceConverter):
                 high = float("inf")
                 shape = (sh,)
                 SpaceType = spaces.Box
-                if attr_nm == "prod_p":
+                if attr_nm == "gen_p" or attr_nm == "gen_p_before_curtail":
                     low = observation_space.gen_pmin
-                    high = observation_space.gen_pmax*1.2  # because of the slack bus... # TODO
+                    high = observation_space.gen_pmax * 1.2  # because of the slack bus... # TODO
                     shape = None
-                elif attr_nm == "prod_v" or attr_nm == "load_v" or attr_nm == "v_or" or attr_nm == "v_ex":
+                elif attr_nm == "gen_v" or attr_nm == "load_v" or attr_nm == "v_or" or attr_nm == "v_ex":
                     # voltages can't be negative
                     low = 0.
                 elif attr_nm == "a_or" or attr_nm == "a_ex" or attr_nm == "rho":
@@ -175,6 +188,17 @@ class GymObservationSpace(_BaseGymSpaceConverter):
                                      -observation_space.gen_pmax)
                     high = np.maximum(-observation_space.gen_pmin,
                                       +observation_space.gen_pmax)
+                elif attr_nm == "storage_power" or attr_nm == "storage_power_target":
+                    low = - observation_space.storage_max_p_prod
+                    high = observation_space.storage_max_p_absorb
+                elif attr_nm == "storage_charge":
+                    low = np.zeros(observation_space.n_storage, dtype=dt_float)
+                    high = observation_space.storage_Emax
+                elif attr_nm == "curtailment" or attr_nm == "curtailment_limit":
+                    low = 0.
+                    high = 1.0
+                # curtailment, curtailment_limit, gen_p_before_curtail
+
                 my_type = SpaceType(low=low, high=high, shape=shape, dtype=dt)
 
             if my_type is None:
@@ -219,4 +243,4 @@ class GymObservationSpace(_BaseGymSpaceConverter):
         return self._base_to_gym(self.spaces.keys(),
                                  grid2op_observation,
                                  dtypes={k: self.spaces[k].dtype for k in self.spaces}
-                                 )
\ No newline at end of file
+                                 )
