@@ -8,6 +8,7 @@
 
 import warnings
 import tempfile
+import json
 import pdb
 
 from grid2op.tests.helper_path_test import *
@@ -19,13 +20,12 @@ from grid2op.Chronics import Multifolder
 from grid2op.Reward import L2RPNReward
 from grid2op.Backend import PandaPowerBackend
 from grid2op.MakeEnv import make
+from grid2op.Runner.Runner import _aux_one_process_parrallel
 from grid2op.Runner import Runner
 from grid2op.dtypes import dt_float
 from grid2op.Agent import RandomAgent
 from grid2op.Episode import EpisodeData
-import json
-
-import warnings
+from grid2op.Observation import CompleteObservation
 warnings.simplefilter("error")
 
 
@@ -35,7 +35,9 @@ class TestRunner(HelperTests):
         self.path_chron = PATH_ADN_CHRONICS_FOLDER
         self.parameters_path = None
         self.max_iter = 10
-        self.real_reward = dt_float(199.99800)
+        # self.real_reward = dt_float(199.99800)
+        self.real_reward = dt_float(179.99818)
+        self.all_real_rewards = [19.999783, 19.999786, 19.999784, 19.999794, 19.9998,   19.999804, 19.999804, 19.999817, 19.999823,  0.      ]
         self.names_chronics_to_backend = {"loads": {"2_C-10.61": 'load_1_0', "3_C151.15": 'load_2_1',
                                                     "14_C63.6": 'load_13_2', "4_C-9.47": 'load_3_3',
                                                     "5_C201.84": 'load_4_4',
@@ -57,48 +59,95 @@ class TestRunner(HelperTests):
                                           }
         self.gridStateclass = Multifolder
         self.backendClass = PandaPowerBackend
-        self.runner = Runner(init_grid_path=self.init_grid_path,
-                             path_chron=self.path_chron,
-                             parameters_path=self.parameters_path,
-                             names_chronics_to_backend=self.names_chronics_to_backend,
-                             gridStateclass=self.gridStateclass,
-                             backendClass=self.backendClass,
-                             rewardClass=L2RPNReward,
-                             max_iter=self.max_iter,
-                             name_env="test_runner_env")
+        with warnings.catch_warnings():
+            warnings.filterwarnings("ignore")  # silence the warning about missing layout
+            self.runner = Runner(init_grid_path=self.init_grid_path,
+                                 path_chron=self.path_chron,
+                                 parameters_path=self.parameters_path,
+                                 names_chronics_to_backend=self.names_chronics_to_backend,
+                                 gridStateclass=self.gridStateclass,
+                                 backendClass=self.backendClass,
+                                 rewardClass=L2RPNReward,
+                                 max_iter=self.max_iter,
+                                 name_env="test_runner_env")
 
-    def test_one_episode(self):
-        _, cum_reward, timestep = self.runner.run_one_episode(max_iter=self.max_iter)
-        assert int(timestep) == self.max_iter
-        assert np.abs(cum_reward - self.real_reward) <= self.tol_one
+    # def test_one_episode(self):  # tested in the runner fast
+    # def test_one_episode_detailed(self):  # tested in the runner fast
+    # def test_2episode(self):  # tested in the runner fast
+    # def test_init_from_env(self):  # tested in the runner fast
+    # def test_seed_seq(self):  # tested in the runner fast
+    # def test_seed_par(self):  # tested in the runner fast
 
     def test_one_process_par(self):
         with warnings.catch_warnings():
             warnings.filterwarnings("ignore")
-            res = Runner._one_process_parrallel(self.runner, [0], 0, None, None, self.max_iter)
+            res = _aux_one_process_parrallel(self.runner, [0], 0, None, None, self.max_iter)
         assert len(res) == 1
         _, el1, el2, el3, el4 = res[0]
         assert el1 == "1"
-        assert np.abs(el2 - 199.9980010986328) <= self.tol_one
+        assert np.abs(el2 - self.real_reward) <= self.tol_one
         assert el3 == 10
         assert el4 == 10
 
-    def test_2episode(self):
-        res = self.runner._run_sequential(nb_episode=2, max_iter=self.max_iter)
-        assert len(res) == 2
-        for i, _, cum_reward, timestep, total_ts in res:
-            assert int(timestep) == self.max_iter
-            assert np.abs(cum_reward - self.real_reward) <= self.tol_one
-
     def test_2episode_2process(self):
         with warnings.catch_warnings():
             warnings.filterwarnings("ignore")
-            res = self.runner._run_parrallel(nb_episode=2, nb_process=2, max_iter=self.max_iter)
+            res = self.runner._run_parrallel(nb_episode=2,
+                                             nb_process=2,
+                                             max_iter=self.max_iter)
         assert len(res) == 2
         for i, _, cum_reward, timestep, total_ts in res:
             assert int(timestep) == self.max_iter
             assert np.abs(cum_reward - self.real_reward) <= self.tol_one
 
+    def test_2episode_2process_detailed(self):
+        with warnings.catch_warnings():
+            warnings.filterwarnings("ignore")
+            res = self.runner.run(nb_episode=2,
+                                  nb_process=2,
+                                  max_iter=self.max_iter,
+                                  add_detailed_output=True)
+        assert len(res) == 2
+        for i, _, cum_reward, timestep, total_ts, episode_data in res:
+            assert int(timestep) == self.max_iter
+            assert np.abs(cum_reward - self.real_reward) <= self.tol_one
+            for j in range(len(self.all_real_rewards)):
+                assert np.abs(episode_data.rewards[j] - self.all_real_rewards[j]) <= self.tol_one
+
+    def test_multiprocess_windows_no_fail(self):
+        """test that i can run multiple times parallel run of the same env (breaks on windows)"""
+        nb_episode = 2
+        with warnings.catch_warnings():
+            warnings.filterwarnings("ignore")
+            with make("rte_case5_example", test=True) as env:
+                f = tempfile.mkdtemp()
+                runner_params = env.get_params_for_runner()
+                runner = Runner(**runner_params)
+                res1 = runner.run(path_save=f,
+                                  nb_episode=nb_episode,
+                                  nb_process=2,
+                                  max_iter=self.max_iter)
+                res2 = runner.run(path_save=f,
+                                  nb_episode=nb_episode,
+                                  nb_process=1,
+                                  max_iter=self.max_iter)
+                res3 = runner.run(path_save=f,
+                                  nb_episode=nb_episode,
+                                  nb_process=2,
+                                  max_iter=self.max_iter)
+        test_ = set()
+        for id_chron, name_chron, cum_reward, nb_time_step, max_ts in res1:
+            test_.add(name_chron)
+        assert len(test_) == nb_episode
+        test_ = set()
+        for id_chron, name_chron, cum_reward, nb_time_step, max_ts in res2:
+            test_.add(name_chron)
+        assert len(test_) == nb_episode
+        test_ = set()
+        for id_chron, name_chron, cum_reward, nb_time_step, max_ts in res3:
+            test_.add(name_chron)
+        assert len(test_) == nb_episode
+
     def test_complex_agent(self):
         nb_episode = 4
         with warnings.catch_warnings():
@@ -116,15 +165,6 @@ class TestRunner(HelperTests):
             test_.add(name_chron)
         assert len(test_) == nb_episode
 
-    def test_init_from_env(self):
-        with warnings.catch_warnings():
-            warnings.filterwarnings("ignore")
-            with make("rte_case14_test", test=True) as env:
-                runner = Runner(**env.get_params_for_runner())
-        res = runner.run(nb_episode=1, max_iter=self.max_iter)
-        for i, _, cum_reward, timestep, total_ts in res:
-            assert int(timestep) == self.max_iter
-
     def test_init_from_env_with_other_reward(self):
         with warnings.catch_warnings():
             warnings.filterwarnings("ignore")
@@ -134,24 +174,6 @@ class TestRunner(HelperTests):
         for i, _, cum_reward, timestep, total_ts in res:
             assert int(timestep) == self.max_iter
 
-    def test_seed_seq(self):
-        with warnings.catch_warnings():
-            warnings.filterwarnings("ignore")
-            with make("rte_case14_test", test=True) as env:
-                runner = Runner(**env.get_params_for_runner())
-        res = runner.run(nb_episode=1, max_iter=self.max_iter, env_seeds=[1], agent_seeds=[2])
-        for i, _, cum_reward, timestep, total_ts in res:
-            assert int(timestep) == self.max_iter
-
-    def test_seed_par(self):
-        with warnings.catch_warnings():
-            warnings.filterwarnings("ignore")
-            with make("rte_case14_test", test=True) as env:
-                runner = Runner(**env.get_params_for_runner())
-        res = runner.run(nb_episode=2, nb_process=2, max_iter=self.max_iter, env_seeds=[1, 2], agent_seeds=[3, 4])
-        for i, _, cum_reward, timestep, total_ts in res:
-            assert int(timestep) == self.max_iter
-
     def test_seed_properly_set(self):
         class TestSuitAgent(RandomAgent):
             def __init__(self, *args, **kwargs):
@@ -220,34 +242,50 @@ class TestRunner(HelperTests):
         for i, _, cum_reward, timestep, total_ts in res:
             assert int(timestep) == 2*self.max_iter
 
-    def _aux_backward(self, base_path, g2op_version):
-        episode_studied = EpisodeData.list_episode(os.path.join(base_path, g2op_version))
+    def _aux_backward(self, base_path, g2op_version_txt, g2op_version):
+        episode_studied = EpisodeData.list_episode(os.path.join(base_path, g2op_version_txt))
         for base_path, episode_path in episode_studied:
+            assert 'curtailment' in CompleteObservation.attr_list_vect, f"error after the legacy version " \
+                                                                        f"{g2op_version}"
             this_episode = EpisodeData.from_disk(base_path, episode_path)
-            with open(os.path.join(os.path.join(base_path, episode_path), "episode_meta.json"), "r",
+            assert 'curtailment' in CompleteObservation.attr_list_vect, f"error after the legacy version " \
+                                                                        f"{g2op_version}"
+            full_episode_path = os.path.join(base_path, episode_path)
+            with open(os.path.join(full_episode_path, "episode_meta.json"), "r",
                       encoding="utf-8") as f:
                 meta_data = json.load(f)
             nb_ts = int(meta_data["nb_timestep_played"])
             try:
                 assert len(this_episode.actions) == nb_ts, f"wrong number of elements for actions for version " \
-                                                           f"{g2op_version}: {len(this_episode.actions)} vs {nb_ts}"
+                                                           f"{g2op_version_txt}: {len(this_episode.actions)} vs {nb_ts}"
                 assert len(this_episode.observations) == nb_ts + 1, f"wrong number of elements for observations " \
-                                                                    f"for version {g2op_version}: " \
+                                                                    f"for version {g2op_version_txt}: " \
                                                                     f"{len(this_episode.observations)} vs {nb_ts}"
                 assert len(this_episode.env_actions) == nb_ts, f"wrong number of elements for env_actions for " \
-                                                               f"version {g2op_version}: " \
+                                                               f"version {g2op_version_txt}: " \
                                                                f"{len(this_episode.env_actions)} vs {nb_ts}"
-            except:
-                import pdb
-                pdb.set_trace()
+            except Exception as exc_:
+                raise exc_
+
+            if g2op_version <= "1.4.0":
+                assert EpisodeData.get_grid2op_version(full_episode_path) == "<=1.4.0", \
+                    "wrong grid2op version stored (grid2op version <= 1.4.0)"
+            elif g2op_version == "test_version":
+                assert EpisodeData.get_grid2op_version(full_episode_path) == grid2op.__version__, \
+                    "wrong grid2op version stored (test_version)"
+            else:
+                assert EpisodeData.get_grid2op_version(full_episode_path) == g2op_version, \
+                    "wrong grid2op version stored (>=1.5.0)"
 
     def test_backward_compatibility(self):
-        backward_comp_version = ["1.0.0", "1.1.0", "1.1.1", "1.2.0", "1.2.1", "1.2.2", "1.2.3", "1.3.0"]
-        curr_version = "current_version"
+        backward_comp_version = ["1.0.0", "1.1.0", "1.1.1", "1.2.0", "1.2.1", "1.2.2", "1.2.3", "1.3.0", "1.3.1",
+                                 "1.4.0", "1.5.0"]
+        curr_version = "test_version"
+        assert 'curtailment' in CompleteObservation.attr_list_vect, "error at the beginning"
         with warnings.catch_warnings():
             warnings.filterwarnings("ignore")
             with make("rte_case5_example", test=True) as env, \
-                 tempfile.TemporaryDirectory() as path:
+                    tempfile.TemporaryDirectory() as path:
                 runner = Runner(**env.get_params_for_runner(), agentClass=RandomAgent)
                 runner.run(nb_episode=2,
                            path_save=os.path.join(path, curr_version),
@@ -256,11 +294,27 @@ class TestRunner(HelperTests):
                            env_seeds=[1, 0],
                            agent_seeds=[42, 69])
                 # check that i can read this data generate for this runner
-                self._aux_backward(path, curr_version)
+                self._aux_backward(path, curr_version, curr_version)
+
+        assert 'curtailment' in CompleteObservation.attr_list_vect, "error after the first runner"
+
+        # check that it raises a warning if loaded on the compatibility version
+        grid2op_version = backward_comp_version[0]
+        with self.assertWarns(UserWarning):
+            self._aux_backward(PATH_PREVIOUS_RUNNER,
+                               f"res_agent_{grid2op_version}",
+                               grid2op_version)
 
         for grid2op_version in backward_comp_version:
             # check that i can read previous data stored from previous grid2Op version
-            self._aux_backward(PATH_PREVIOUS_RUNNER, f"res_agent_{grid2op_version}")
+            # can be loaded properly
+            with warnings.catch_warnings():
+                warnings.filterwarnings("ignore")
+                self._aux_backward(PATH_PREVIOUS_RUNNER,
+                                   f"res_agent_{grid2op_version}",
+                                   grid2op_version)
+            assert 'curtailment' in CompleteObservation.attr_list_vect, f"error after the legacy version " \
+                                                                        f"{grid2op_version}"
 
 
 if __name__ == "__main__":
